"""Test the functionality of conf2spec part of the tool.
conf2spec can do a bit more that the fully automatic generation of spec files.
The tests test both the automatically generated files and the custom
configurations.
"""

from pathlib import Path
import pytest

from pyp2spec import conf2spec


@pytest.fixture
def config_dir():
    return "tests/test_configs/"


def test_manual_brs_are_loaded(config_dir):
    config_path = config_dir + "customized_boutdata.conf"
    config = conf2spec.ConfigFile(config_path)
    expected = [
        "python3dist(setuptools)",
        "python3dist(setuptools-scm[toml]) >= 3.4",
        "python3dist(setuptools-scm-git-archive)",
        "python3dist(pytest)",
    ]
    assert config.get_list("manual_build_requires") == expected


def test_no_manual_brs(config_dir):
    config_path = config_dir + "customized_click.conf"
    config = conf2spec.ConfigFile(config_path)
    assert config.get_list("manual_build_requires") == []


def test_long_description_is_split(config_dir):
    config_path = config_dir + "customized_jupyter-packaging.conf"
    config = conf2spec.ConfigFile(config_path)
    expected = "This package contains utilities for making Python packages with and without\naccompanying JavaScript packages."
    assert conf2spec.wrap_description(config) == expected


@pytest.mark.parametrize(
    ("conf", "expected"), [
        ("customized_boutdata.conf", "%pytest"),
        ("customized_click.conf", "%tox"),
        ("customized_aionotion.conf", ""),
        ("customized_markdown-it-py.conf", "%pytest -k 'not test_file and \\\nnot test_linkify'")
    ]
)
def test_check_is_generated(config_dir, conf, expected):
    config_path = config_dir + conf
    config = conf2spec.ConfigFile(config_path)
    assert conf2spec.generate_check(config) == expected


@pytest.mark.parametrize(
    ("conf", "expected"), [
        ("customized_boutdata.conf", True),
        ("customized_markdown-it-py.conf", False),
    ]
)
def test_top_level_flag_is_loaded(config_dir, conf, expected):
    config_path = config_dir + conf
    config = conf2spec.ConfigFile(config_path)
    assert config.get_bool("test_top_level") == expected


@pytest.mark.parametrize(
    ("conf", "expected"), [
        ("customized_boutdata.conf", ["LICENSE"]),
        ("customized_markdown-it-py.conf", ["LICENSE", "LICENSE.markdown-it"]),
        ("customized_aionotion.conf", []),
    ]
)
def test_license_files_are_loaded(config_dir, conf, expected):
    config_path = config_dir + conf
    config = conf2spec.ConfigFile(config_path)
    assert config.get_list("license_files") == expected


@pytest.mark.parametrize(
    ("conf", "expected"), [
        ("customized_click.conf", ["README.rst", "CHANGES.rst"]),
        ("customized_markdown-it-py.conf", ["README.md"]),
        ("customized_aionotion.conf", []),
    ]
)
def test_doc_files_are_loaded(config_dir, conf, expected):
    config_path = config_dir + conf
    config = conf2spec.ConfigFile(config_path)
    assert config.get_list("doc_files") == expected


@pytest.mark.parametrize(
    ("conf", "expected"), [
        ("customized_click.conf", []),
        ("customized_markdown-it-py.conf", ["markdown-it"]),
    ]
)
def test_binary_files_are_loaded(config_dir, conf, expected):
    config_path = config_dir + conf
    config = conf2spec.ConfigFile(config_path)
    assert config.get_list("binary_files") == expected


@pytest.mark.parametrize(
    ("conf", "expected"), [
        ("customized_click.conf", False),
        ("customized_tornado.conf", True),
    ]
)
def test_archful_flag_is_loaded(config_dir, conf, expected):
    config_path = config_dir + conf
    config = conf2spec.ConfigFile(config_path)
    assert config.get_bool("archful") is expected


@pytest.mark.parametrize(
    "conf", [
        "default_python-click.conf",
        "customized_markdown-it-py.conf",
        "customized_python-sphinx.conf",
        "default_python-numpy.conf",
        "default_python3.9-pello.conf",
    ]
)
def test_default_generated_specfile(file_regression, config_dir, conf):
    # Run the conf2spec converter
    rendered_file = conf2spec.create_spec_file(config_dir + conf)

    # Compare the results
    with open(rendered_file, "r", encoding="utf-8") as rendered_f:
        rendered = rendered_f.read()
    try:
        file_regression.check(
            rendered,
            fullpath=f"tests/expected_specfiles/{rendered_file}",
        )
    finally:
        # Cleanup - remove created file
        Path.unlink(Path(rendered_file))



from functools import wraps
import re
import sys

import click
import requests
import tomli_w

from packaging.requirements import Requirement

from pyp2spec.rpmversion import RpmVersion
from pyp2spec.license_processor import classifiers_to_spdx_identifiers
from pyp2spec.license_processor import license_keyword_to_spdx_identifiers, good_for_fedora
from pyp2spec.utils import Pyp2specError


class NoLicenseDetectedError(Pyp2specError):
    """Raised when there's no valid license detected"""


class SdistNotFoundError(Pyp2specError):
    """Raised when there's no sdist file in the PyPI metadata"""


class PackageNotFoundError(Pyp2specError):
    """Raised when there's no such package name on PyPI"""


class PypiPackage:
    """Store and process the package data obtained from PyPI.

    It's possible to create the object these ways:
    - providing a package name.
    A request is made to PyPI general project API to find the latest available version.
    Another request is made to the PyPI package version API to obtain all
    package data related to that particular distribution.
    - providing a package name and a version.
    A request is made to the PyPI package version API to obtain all
    package data related to that particuler distribution.
    - providing a package name, version and custom package metadata
    (dictionary with the same structure as PyPI package version API).
    No requests to PyPI are made, the package metadata serve as the source
    to create a PypiPackage object.
    This is only relevant for testing.
    """

    def __init__(self, package_name, *, version=None, package_metadata=None, session=None):
        self.package_name = package_name
        self._session = session or requests.Session()
        self.version = version or self._get_version_from_package_metadata()
        # package_metadata - custom dictionary with package metadata - used for testing
        # in the typical app run it's not set,
        # meaning we jump to the other sources package metadata data (eg. PyPI)
        self.package_data = package_metadata or self._get_package_version_metadata()
        self.sdist_filename = None

    @property
    def pypi_name(self):
        """Return normalized PyPI package name (as defined in PEP 503).
        The resulting string better conforms with Fedora's Packaging Guidelines.
        """

        return self.normalize(self.package_data["info"]["name"])

    def normalize(self, package_name):
        """Normalize given package name as defined in PEP 503"""

        return re.sub(r"[-_.]+", "-", package_name).lower()

    def _get_from_url(self, url, error_str):
        response = self._session.get(url)
        if not response.ok:
            raise PackageNotFoundError(error_str)
        return response.json()

    def _get_package_project_metadata(self):
        pkg_index = f"https://pypi.org/pypi/{self.package_name}/json"
        error_str = f"Package `{self.package_name}` was not found on PyPI"
        return self._get_from_url(pkg_index, error_str)

    def _get_version_from_package_metadata(self):
        package_metadata = self._get_package_project_metadata()
        return package_metadata["info"]["version"]

    def _get_package_version_metadata(self):
        pkg_index = f"https://pypi.org/pypi/{self.package_name}/{self.version}/json"
        error_str = f"Package `{self.package_name}` or version `{self.version}` was not found on PyPI"
        return self._get_from_url(pkg_index, error_str)

    def python_name(self, *, python_alt_version=None):
        """Create a component name for the specfile.

        Prepend the name with 'python' (unless it already starts with it).
        Add the Python alternative version, if it's defined.

        Valid outcomes:
        package name: foo, python_alt_version: 3.11
        -> python3.11-foo

        package name: foo, python_alt_version: None
        -> python-foo

        package name: python-foo, python_alt_version: 3.12
        -> python3.12-foo

        package name: python-foo, python_alt_version: None
        -> python-foo
        """

        alt_version = "" if python_alt_version is None else python_alt_version
        if self.pypi_name.startswith("python"):
            return self.pypi_name.replace("python", f"python{alt_version}")
        return f"python{alt_version}-{self.pypi_name}"

    def source(self):
        """Return valid pypi_source RPM macro.

        %pypi_source takes three optional arguments:
        <name>, <version> and <file_extension>.
        <name> is always passed, it's spelled as the name of the archive file:
        "%{pypi_source foo}".
        <version> is passed if PyPI's and RPM's version strings differ:
        "%{pypi_source foo 1.2-3}". If they're the same and the file extension
        is not zip, version is not passed.
        If archive is a zip file, %{pypi_source} must take all three args:
        "%{pypi_source foo %{version} zip}", "{pypi_source foo 1.2-3 zip}"
        """
        is_zip = self.is_zip_archive()
        version_str = self.pypi_version_or_macro()

        source_macro_args = self.archive_name()

        if is_zip:
            source_macro_args += f" {version_str} zip"
        else:
            if version_str == self.version:
                source_macro_args += f" {version_str}"

        return "%{pypi_source " + source_macro_args + "}"

    def pypi_version_or_macro(self):
        """If PyPI and RPM version's strings are the same, there's no need to
        duplicate them across the spec file.
        Return '%{version}' as a reference to the RPM's version string.
        If they are different, both variants of string need to be used.
        In such case return version string as from PyPI.
        """
        rpm_version = convert_version_to_rpm_scheme(self.version)
        if self.version == rpm_version:
            return "%{version}"
        return self.version

    def filter_license_classifiers(self):
        """Return the list of license classifiers defined for the package.

        Filter out the parent categories `OSI-/DFSG Approved` which don't have any meaning.
        """

        return  [
            c for c in self.package_data["info"]["classifiers"]
            if (
                c.startswith("License")
                and c not in ("License :: OSI Approved", "License :: DFSG approved")
            )
        ]

    def transform_to_spdx(self):
        """Return SPDX identifiers and expression based on the found
        package license metadata (classifiers or license keyword).
        
        If multiple identifiers are found, create an expression that's the safest option (with AND as joining operator).
        Raise NoLicenseDetectedError if no valid SPDX identifiers are found.
        """

        if (license_classifiers := self.filter_license_classifiers()):
            identifiers = classifiers_to_spdx_identifiers(license_classifiers)
            if identifiers:
                expression = " AND ".join(identifiers)
                return (identifiers, expression)

        license_keyword = self.package_data["info"]["license"]
        identifiers = license_keyword_to_spdx_identifiers(license_keyword)

        # No more options to detect licenses left, hence explicit fail
        if not identifiers:
            raise NoLicenseDetectedError("No valid license detected.")
        return (identifiers, license_keyword)

    def license(self, *, check_compliance=False, licenses_dict=None):
        """Return the license string from package metadata.

        If `check_compliance` is set to True, check each of the found
        SPDX identifiers against the Fedora allowed licenses.
        This isn't a 100% reliable solution and in case of ambiguous results,
        the license is discarded as invalid.
        If the compliant license can't be determined, raise NoLicenseDetectedError.
        """

        identifiers, expression = self.transform_to_spdx()
        if check_compliance:
            is_compliant, bad_identifiers = good_for_fedora(identifiers, session=self._session, licenses_dict=licenses_dict)
            if not is_compliant:
                if bad_identifiers:
                    err_string = "The detected licenses: `{0}` aren't allowed in Fedora."
                    click.secho(err_string.format(", ".join(bad_identifiers), fg="red"))
                raise NoLicenseDetectedError("Could not create a compliant license field.")

        return expression

    def summary(self):
        summary = self.package_data["info"]["summary"]
        # summary may not be filled in the upstream data or it can consist of
        # more than one line - in both cases use the autogenerated string
        if not summary or len(summary.split("\n")) > 1:
            summary = "..."
        return summary

    def project_url(self):
        try:
            return self.package_data["info"]["project_urls"]["Homepage"]
        # It may happen that no project_urls nor Homepage are listed with the
        # project - in this case fall back to the safe PyPI URL
        except (KeyError, TypeError):
            return self.package_data["info"]["package_url"]

    def is_archful(self):
        """Determine if package is archful by checking the wheel filenames.

        Wheel name consists of defined fields, one of them being an abi tag.
        Example abi tags:
        - click-0.2-py2.py3-none-any.whl -> "none"
        - cryptography-2.2-cp34-abi3-manylinux1_x86_64.whl -> "abi3"
        If the value is "none", wheel was not built for a specific architecture,
        probably containing pure Python modules.
        Other values indicate build for an architechture, which can mean
        the presence of compiled extensions.
        Packages can publish multiple wheels, the pure-Python alongside the compiled ones.
        For our purposes, if we find at least one wheel with an abi tag different that "none",
        we consider the package archful.
        The compiled extensions bring optimizations and in Fedora,
        it is generally encouraged to bring in the optional features of the packages.
        """

        for entry in self.package_data["urls"]:
            if entry["packagetype"] == "bdist_wheel":
                abi_tag = entry["filename"].split("-")[-2]
                if abi_tag != "none":
                    return True
        # all of the found wheel names had 'none' as abi_tag
        return False

    def find_and_save_sdist_filename(self):
        """Save the given's package version sdist name for further processing.

        Quit the script if not found (bdists can't be processed).
        """
        for entry in self.package_data["urls"]:
            if entry["packagetype"] == "sdist":
                self.sdist_filename = entry["filename"]
                return
        raise SdistNotFoundError("sdist not found.")

    def is_zip_archive(self):
        """Return True if archive is a zip file, False otherwise.

        The archive format encouraged in PEP 517 is tar.gz, zip is discouraged,
        some projects however still use it.
        If so, it has to be explicitly declared as an argument of %{pypi_source}.
        """
        if self.sdist_filename is None:
            self.find_and_save_sdist_filename()
        if self.sdist_filename.endswith(".zip"):
            return True
        return False

    def archive_name(self):
        """Return the package name as spelled in the archive file.

        According to:
        https://packaging.python.org/en/latest/specifications/source-distribution-format/#source-distribution-file-name
        the `de facto` standard for sdist naming is '{name}-{version}.tar.gz'.
        The format is not standardised, so extract the {name} as defined by upstream.
        Example: "My-package_Archive-1.0.4-12.tar.gz" -> "My-package_Archive"
        """
        if self.sdist_filename is None:
            self.find_and_save_sdist_filename()

        edited_sdist_filename = self.sdist_filename
        # First, strip the suffix
        for suffix in (".tar.gz", ".zip"):
            edited_sdist_filename = edited_sdist_filename.removesuffix(suffix)
        # Second, get rid of the version string and the delimiter "-"
        archive_name = edited_sdist_filename.replace("-" + self.version, "")
        return archive_name

    def extras(self):
        """Return the sorted list of the found extras names.

        Packages define extras explicitly via `Provides-Extra` and
        indirectly via `Requires-Dist` metadata.
        PyPI metadata doesn't provide the first one, but it is possible to
        derive extras names from the `requires_dist` key.
        Example value of `requires_dist`:
        ["sphinxcontrib-websupport ; extra == 'docs'", "flake8>=3.5.0 ; extra == 'lint'"]
        If package defines an extra with no requirements, we can't detect that.
        """
        extra_from_req = re.compile(r'''\bextra\s+==\s+["']([^"']+)["']''')
        extras = set()
        requires_dist = self.package_data["info"]["requires_dist"]
        if requires_dist is not None:
            for required_dist in requires_dist:
                # packaging.Requirement can parse the markers, but it
                # doesn't provide their string representations,
                # hence we need to use regex to pick them out
                req = Requirement(required_dist)
                if found := re.search(extra_from_req, str(req.marker)):
                    extras.add(found.group(1))
        return sorted(extras)


def get_description(package):
    """Return a default package description."""

    return f"This is package '{package}' generated automatically by pyp2spec."


def convert_version_to_rpm_scheme(version):
    """If version follows PEP 440, return its value converted to RPM scheme.

    PEP 440: https://www.python.org/dev/peps/pep-0440/
    If the package uses a different versioning scheme (i.e. LegacyVersion),
    the returned value will be the same as the input one.
    Such value may or may not work with RPM.
    Automatic conversion of the LegacyVersions is not feasible, as stated in:
    https://lists.fedoraproject.org/archives/list/python-devel@lists.fedoraproject.org/message/5MGEHMTKOKR5U7ACIMUDRBKMSP6Y5NQD/
    """
    return str(RpmVersion(version))


def is_package_name(package):
    """Least-effort check whether `package` is a package name or URL.
    Canonical package names can't contain '/'.
    """

    return not "/" in package


def create_config_contents(
    package,
    description=None,
    version=None,
    summary=None,
    session=None,
    license=None,
    compliant=False,
    top_level=False,
    python_alt_version=None,
):
    """Use `package` and provided kwargs to create the whole config contents.
    Return contents dictionary.
    """
    contents = {}

    # a package name was given as the `package`, look for it on PyPI
    if is_package_name(package):
        click.secho(f"Assuming '{package}' is a package name", fg="yellow")
        pkg = PypiPackage(package, version=version, session=session)
        click.secho(f"Querying PyPI for package '{package}'", fg="cyan")
    # a URL was given as the `package`
    else:
        raise NotImplementedError("pyp2spec can't currently handle URLs.")

    if description is None:
        description = get_description(package)
        click.secho(f"Assuming --description={description}", fg="yellow")

    if summary is None:
        summary = pkg.summary()
        click.secho(f"Assuming --summary={summary}", fg="yellow")

    if version is None:
        click.secho(f"Assuming PyPI --version={pkg.version}", fg="yellow")

    if license is None:
        license = pkg.license(check_compliance=compliant)
        click.secho(f"Assuming --license={license}", fg="yellow")

    if top_level:
        click.secho("Only top-level modules will be checked", fg="yellow")
        contents["test_top_level"] = True

    if archful := pkg.is_archful():
        click.secho("Package is archful - you may need to specify additional build requirements", fg="magenta")

    if python_alt_version is not None:
        click.secho(f"Assuming build for Python: {python_alt_version}", fg="yellow")
        contents["python_alt_version"] = python_alt_version

    contents["archful"] = archful
    contents["description"] = description
    contents["summary"] = summary
    contents["version"] = convert_version_to_rpm_scheme(pkg.version)
    contents["pypi_version"] = pkg.pypi_version_or_macro()
    contents["license"] = license
    contents["pypi_name"] = pkg.pypi_name
    contents["python_name"] = pkg.python_name(python_alt_version=python_alt_version)
    contents["url"] = pkg.project_url()
    contents["source"] = pkg.source()
    contents["archive_name"] = pkg.archive_name()
    contents["extras"] = pkg.extras()

    return contents


def save_config(contents, output=None):
    """Write config file to a given destination.
    If none is provided, save it to current directory with package name as file name.
    Return the saved file name.
    """
    if not output:
        package = contents["python_name"]
        output = f"./{package}.conf"
    with open(output, "wb") as f:
        click.secho(f"Saving configuration file to '{output}'", fg="yellow")
        tomli_w.dump(contents, f, multiline_strings=True)
    click.secho("Configuration file was saved successfully", fg="green")
    return output


def create_config(options):
    """Create and save config file."""

    contents = create_config_contents(
        options["package"],
        description=options["description"],
        version=options["version"],
        summary=options["summary"],
        license=options["license"],
        compliant=options["fedora_compliant"],
        top_level=options["top_level"],
        python_alt_version=options["python_alt_version"]
    )
    return save_config(contents, options["config_output"])


def pypconf_args(func):
    @click.argument("package")
    @click.option(
        "--config-output", "-c",
        help="Provide custom output for configuration file",
    )
    @click.option(
        "--description", "-d",
        help="Provide description for the package",
    )
    @click.option(
        "--version", "-v",
        help="Provide package version to query PyPI for, default: latest",
    )
    @click.option(
        "--summary", "-s",
        help="Provide custom package summary",
    )
    @click.option(
        "--license", "-l",
        help="Provide license name",
    )
    @click.option(
        "--fedora-compliant", is_flag=True,
        help="Check whether license is compliant with Fedora",
    )
    @click.option(
        "--top-level", "-t", is_flag=True,
        help="Test only top-level modules in %check",
    )
    @click.option(
        "--python-alt-version", "-p",
        help="Provide specific Python version to build for, e.g 3.11",
    )
    @wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper

@click.command()
@pypconf_args
def main(**options):
    try:
        create_config(options)
    except (Pyp2specError, NotImplementedError) as exc:
        click.secho(f"Fatal exception occurred: {exc}", fg="red")
        sys.exit(1)

if __name__ == "__main__":
    main()



import sys

import click

from pyp2spec.pyp2conf import create_config, pypconf_args
from pyp2spec.conf2spec import create_spec_file
from pyp2spec.utils import Pyp2specError


@click.command()
@pypconf_args
@click.option(
    "--spec-output", "-o",
    help="Provide custom output where spec file will be saved",
)
def main(**options):
    click.secho("Generating configuration file", fg="cyan")
    try:
        config_file = create_config(options)
        click.secho("Generating spec file", fg="cyan")
        create_spec_file(config_file, options["spec_output"])
    except (Pyp2specError, NotImplementedError) as exc:
        click.secho(f"Fatal exception occurred: {exc}", fg="red")
        sys.exit(1)
    click.secho("Done", fg="green")


if __name__ == "__main__":
    main()



import sys

from importlib.resources import files
from textwrap import fill

import click

try:
    import tomllib
except ImportError:
    import tomli as tomllib

from jinja2 import Template

from pyp2spec.utils import Pyp2specError


TEMPLATE_FILENAME = "template.spec"
ADDITIONAL_BUILD_REQUIRES_ARCHFUL = ["gcc"]
ADDITIONAL_BUILD_REQUIRES_NOARCH = []


class ConfigError(Pyp2specError):
    """Raised when a config value has got a wrong type"""


class ConfigFile:
    """Load configuration file and return its validated values."""

    def __init__(self, filename):
        self.filename = filename
        self.contents = self.load_configuration

    @property
    def load_configuration(self):
        """Return loaded TOML configuration file contents."""

        with open(self.filename, "rb") as configuration_file:
            loaded_contents = tomllib.load(configuration_file)

        return loaded_contents

    def get_string(self, key):
        """Return a value for given key. Validate the value is a string.
        Raise ConfigError otherwise."""

        return self._get_value(key, str)

    def get_list(self, key):
        """Return a value for given key. Validate the value is a list.
        Raise ConfigError otherwise."""

        return self._get_value(key, list)

    def get_bool(self, key):
        """Return a value for given key. Validate the value is a boolean.
        Raise ConfigError otherwise."""

        return self._get_value(key, bool)

    def _get_value(self, key, val_type):
        val = self.contents.get(key, val_type())
        if not isinstance(val, val_type):
            raise ConfigError(f"{val} must be a {val_type}")
        return val


def generate_check(config):
    """Generate valid check section.
    If defined in config file, use applicable test macro.
    If no test method was defined, return an empty string."""

    test_method = config.get_string("test_method")
    if test_method == "pytest":
        return generate_pytest(config)
    elif test_method == "tox":
        return generate_tox(config)
    else:
        return ""


def generate_pytest(config):
    """Return valid pytest macro with unwanted tests (if defined)."""

    if unwanted_tests := generate_unwanted_tests(config):
        return f"%pytest {unwanted_tests}"
    return "%pytest"


def generate_tox(config):
    """Return valid tox macro. If additional unwanted are defined,
    raise NotImplementedError."""

    if unwanted_tests := generate_unwanted_tests(config):
        raise NotImplementedError("It's currently impossible to filter out tests with %tox")
    return "%tox"


def generate_unwanted_tests(config):
    """If defined in config file, get the unwanted tests.
    Given the unwanted tests are [a, b], return `-k "not a and\\\nnot b"`."""

    unwanted_tests = config.get_list("unwanted_tests")
    if not unwanted_tests:
        return ""
    else:
        prep_unwanteds = [f"not {test}" for test in unwanted_tests]
        unwanteds_as_str = " and \\\n".join(prep_unwanteds)
        formatted_unwanteds = f"-k '{unwanteds_as_str}'"
        return formatted_unwanteds


def list_additional_build_requires(config):
    """Returns a list of additionally defined BuildRequires,

    The list differs for packages that are or aren't archful.
    """
    if config.get_bool("archful"):
        return ADDITIONAL_BUILD_REQUIRES_ARCHFUL
    return ADDITIONAL_BUILD_REQUIRES_NOARCH


def wrap_description(config):
    """Wrap description line to 79 characters at most.

    If the description line length exceeds 79 lines, tools like rpmlint start
    emit warnings. Use newlines to prevent exceeding the max. length.
    Return the modified description string.
    """
    return fill(
        config.get_string("description"),
        width=79,
        break_long_words=False
    )


def python3_pkgversion_or_3(config):
    return "%{python3_pkgversion}" if config.get_string("python_alt_version") else "3"


def fill_in_template(config):
    """Return template rendered with data from config file."""

    with (files("pyp2spec") / TEMPLATE_FILENAME).open("r", encoding="utf-8") as f:
        spec_template = Template(f.read())

    result = spec_template.render(
        additional_build_requires=list_additional_build_requires(config),
        archful=config.get_bool("archful"),
        archive_name=config.get_string("archive_name"),
        binary_files=config.get_list("binary_files"),
        description=wrap_description(config),
        doc_files=" ".join(config.get_list("doc_files")),
        extras=",".join(config.get_list("extras")),
        license_files=" ".join(config.get_list("license_files")),
        license=config.get_string("license"),
        name=config.get_string("pypi_name"),
        python_name=config.get_string("python_name"),
        pypi_version=config.get_string("pypi_version"),
        python_alt_version=config.get_string("python_alt_version"),
        source=config.get_string("source"),
        summary=config.get_string("summary"),
        test_method=generate_check(config),
        test_top_level=config.get_bool("test_top_level"),
        python3_pkgversion=python3_pkgversion_or_3(config),
        url=config.get_string("url"),
        version=config.get_string("version"),
    )

    return result


def save_spec_file(config, output):
    """Save the spec file in the current directory if custom output is not set.
    Return the saved file name."""

    result = fill_in_template(config)
    if output is None:
        output = config.get_string("python_name") + ".spec"
    with open(output, "w", encoding="utf-8") as spec_file:
        spec_file.write(result)
    click.secho(f"Spec file was saved successfully to '{output}'", fg="green")
    return output


def create_spec_file(config_file, spec_output=None):
    """Create and save the generate spec file."""
    config = ConfigFile(config_file)
    return save_spec_file(config, spec_output)


@click.command()
@click.argument("config")
@click.option(
    "--spec-output",
    "-o",
    help="Provide custom output for spec file",
)
def main(config, spec_output):
    try:
        create_spec_file(config, spec_output)
    except (Pyp2specError, NotImplementedError) as exc:
        click.secho(f"Fatal exception occurred: {exc}", fg="red")
        sys.exit(1)

if __name__ == "__main__":
    main()



from importlib.resources import files
import json

import requests
from license_expression import get_spdx_licensing, ExpressionError

from pyp2spec.utils import Pyp2specError


TROVE2FEDORA_MAP = {}
FEDORA_LICENSES = {}


class NoSuchClassifierError(Pyp2specError):
    """Raised when the detected license classifier doesn't exist in pyp2spec's data"""


class InvalidSPDXExpressionError(Pyp2specError):
    """Raised when the assumed SPDX expression is not valid"""


def _load_package_resource(filename):
    with (files("pyp2spec") / filename).open("r", encoding="utf-8") as f:
        return json.load(f)


def _load_from_drive(source_path):
    with open(source_path, "r", encoding="utf-8") as f:
        return json.load(f)


def _load_from_url(url, session=None):
    s = session or requests.Session()
    response = s.get(url)
    response.raise_for_status()
    return response.json()


def classifiers_to_spdx_identifiers(classifiers):
    """Return the list of SPDX identifiers converted from the license classifiers.

    If the conversion of any of the classifiers is not possible, return None.
    Raise KeyError if the source data doesn't contain the found classifier.
    """

    if not TROVE2FEDORA_MAP:
        # Load the file with Python trove classifiers mapped to SPDX identifiers
        TROVE2FEDORA_MAP.update(_load_package_resource("classifiers_to_fedora.json"))

    spdx_identifiers = []
    for classifier in classifiers:
        try:
            fedora_identifier = TROVE2FEDORA_MAP[classifier]
        except KeyError as err:
            err_string = f"{classifier}: Such classifier doesn't exist. " \
            "If you believe that's pyp2spec's error, open an issue at the project's GitHub repo: " \
            "https://github.com/befeleme/pyp2spec"
            raise NoSuchClassifierError(err_string) from err

        # Classifiers that don't map unambiguously to a single Fedora SPDX expression are nulls
        # in the json
        # If the conversion is not possible, return None - meaning that another method of determining the license will be tried
        if fedora_identifier is None:
            return None
        spdx_identifiers.append(fedora_identifier)
    return spdx_identifiers


def license_keyword_to_spdx_identifiers(license_keyword):
    """Return a sorted list of SPDX identifiers extracted from the license_keyword
    
    If no identifiers were parsed out of the license_keyword, return None.
    Raise ValueError if the license_keyword isn't a valid SPDX expression.
    """

    # `Artistic-1.0-Perl` alone is forbidden in Fedora, but the combination is allowed
    # These expressions may be a part of even longer license strings, which is impossible to cover
    # In an unlikely event of such license in the upstream data, raise an explicit exception
    special_cases = [
        "GPL-1.0-or-later OR Artistic-1.0-Perl",
        "GPL-2.0-or-later OR Artistic-1.0-Perl"
    ]
    for case in special_cases:
        if case in license_keyword:
            err_str = f"The detected license contains `{case}` which is handled in a special way in Fedora. " \
            "If you encountered this error, open an issue at https://github.com/befeleme/pyp2spec " \
            "and include the details about the package that causes the issue."
            raise NotImplementedError(err_str)

    licensing = get_spdx_licensing()
    try:
        parsed_license = licensing.parse(license_keyword, validate=True)
        if parsed_license is None:  # license keyword is probably empty
            return None
        # The objects are stored in sets, sort and return as a list
        return sorted(parsed_license.objects)
    except ExpressionError as err:
        raise InvalidSPDXExpressionError(f"Invalid SPDX expression: {license_keyword}") from err


def _load_fedora_licenses(source_path=None, session=None):
    """Load the dictionary of licenses evaluated for Fedora by the Fedora Legal team.

    Try to get them from the hard drive (installed by `fedora-license-data`).
    Fall back to the resources published by Fedora Legal online when the file isn't found.
    """

    fedora_licenses_path = source_path or "/usr/share/fedora-license-data/licenses/fedora-licenses.json"
    try:
        return _load_from_drive(fedora_licenses_path)

    except FileNotFoundError:
        url = "https://gitlab.com/fedora/legal/fedora-license-data/-/jobs/artifacts/main/raw/fedora-licenses.json?job=json"
        return _load_from_url(url, session=session)


def _is_compliant_with_fedora(identifier, fedora_licenses):
    """Return True if the given identifier is "allowed" for Fedora and False if not.

    Fedora allows different types of licenses: "allowed for content", "allowed fonts",
    "allowed firmware", "allowed for documentation" and "allowed" for any use.
    For our purposes, only the generally "allowed" licenses are considered good.
    """

    # fedora_licenses doesn't have useful keys, hence iterate only through values
    # {"0": {
    #     "license": {
    #         "expression": "Rdisc",
    #         "status": ["allowed"],
    #         "url": "https://fedoraproject.org/wiki/Licensing/Rdisc_License"
    #     },
    #     "fedora": {
    #         "legacy-name": ["Rdisc License"],
    #         "legacy-abbreviation": ["Rdisc"]
    #     },
    #     "approved": "yes",
    #     "fedora_abbrev": "Rdisc",
    #     "fedora_name": "Rdisc License",
    #     "spdx_abbrev": "Rdisc"}...}

    for entry in fedora_licenses.values():
        license_info = entry.get("license")
        if license_info is not None:
            if identifier == entry["license"]["expression"]:
                return "allowed" in entry["license"]["status"]
    # No such identifier was found, we assume it's not good for Fedora
    return False


def good_for_fedora(spdx_identifiers, *, licenses_dict=None, session=None):
    """Determine whether all of the given SPDX identifiers are good for Fedora.

    If no `spdx_identifiers` are given, return tuple: (False, []).
    If all of the given identifiers are good, return (True, []),
    otherwise return (False, [<all bad identifiers>]).
    bad_identifiers are returned in the same order as the given spdx_identifiers.
    """

    # populate FEDORA_LICENSES only if they're still empty and
    # no other dictionary with licenses was given to be used here
    if licenses_dict is None and not FEDORA_LICENSES:
        FEDORA_LICENSES.update(_load_fedora_licenses(session=session))
    fedora_licenses = licenses_dict or FEDORA_LICENSES

    if not spdx_identifiers:
        return (False, [])
    bad_identifiers = []
    for spdx_identifier in spdx_identifiers:
        if not _is_compliant_with_fedora(spdx_identifier, fedora_licenses):
            bad_identifiers.append(spdx_identifier)
    if bad_identifiers:
        return (False, bad_identifiers)
    return (True, [])



import pytest

from pyp2spec.license_processor import classifiers_to_spdx_identifiers, license_keyword_to_spdx_identifiers
from pyp2spec.license_processor import _is_compliant_with_fedora, good_for_fedora
from pyp2spec.license_processor import InvalidSPDXExpressionError, NoSuchClassifierError


@pytest.mark.parametrize(
    ("classifiers", "spdx_identifiers"),
    (
        (["License :: OSI Approved :: X.Net License"], ["Xnet"]),  # good
        (["License :: OSI Approved :: X.Net License", "License :: OSI Approved :: Artistic License"], None),  # good & bad
        (["License :: OSI Approved :: Artistic License"], None),  # bad
        (["License :: OSI Approved :: Eiffel Forum License", "License :: OSI Approved :: GNU General Public License (GPL)"], None),  # bad & bad
        (["License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)", "License :: OSI Approved :: ISC License (ISCL)"], ["LGPL-3.0-or-later", "ISC"]),  # good & good
    ),
)
def test_classifiers_to_spdx_identifiers(classifiers, spdx_identifiers):
    assert classifiers_to_spdx_identifiers(classifiers) == spdx_identifiers


@pytest.mark.parametrize(
    ("classifiers"),  (
        ["License :: OSI Approved :: XXXXXX"],
        ["Non-existing-classifier"],
        [""]
    ),
)
def test_conversion_to_spdx_fails_for_invalid_inputs(classifiers):
    with pytest.raises(NoSuchClassifierError):
        classifiers_to_spdx_identifiers(classifiers)


@pytest.mark.parametrize(
    ("expression", "expected"),
    (
        ("LGPL-2.1-only AND MIT AND BSD-2-Clause", ['BSD-2-Clause', 'LGPL-2.1-only', 'MIT']),
        ("MIT AND (LGPL-2.1-or-later OR BSD-3-Clause)", ['BSD-3-Clause', 'LGPL-2.1-or-later', 'MIT']),
        ("(MIT AND (LGPL-2.1+ AND BSD-3-Clause))", ['BSD-3-Clause', 'LGPL-2.1-or-later', 'MIT']),
        ("GPL-3.0-or-later WITH GCC-exception-3.1", ["GPL-3.0-or-later WITH GCC-exception-3.1"]),
        ("GPL-3.0-or-later WITH GCC-exception-3.1 OR Apache-2.0 OR MPL-2.0", ["Apache-2.0", "GPL-3.0-or-later WITH GCC-exception-3.1", "MPL-2.0"]),
        ("", None),
    ),
)
def test_split_expressions_to_identifiers(expression, expected):
    assert license_keyword_to_spdx_identifiers(expression) == expected


def test_perl_license_raises_exception():
    with pytest.raises(NotImplementedError):
        license_keyword_to_spdx_identifiers("MPL-2.0 OR GPL-2.0-or-later OR Artistic-1.0-Perl AND BSD-3-Clause")


@pytest.mark.parametrize(
    ("expression"),
    (
        ("FOO AND bar OR baz WITH EXCEPTION"),
        ("an arbitrary string"),
        ("multiline\nstring"),
        ("BSD BSD AND BSD"),
        ("string"),
    ),
)
def test_invalid_expressions_raise_errors(expression):
    with pytest.raises(InvalidSPDXExpressionError):
        license_keyword_to_spdx_identifiers(expression)



@pytest.mark.parametrize(
    ("identifier", "expected"),
    (
        ("CECILL-C", True),
        ("GPL-3.0-or-later WITH Classpath-exception-2.0", True),
        ("GPL-3.0-or-later WITH GCC-exception-3.1", True),
        ("CC0-1.0", False),  # allowed content license
        ("LicenseRef-LPPL", False),  # allowed font license
        ("EUPL-1.0", False),  # not allowed at all
        ("Bitstream-Vera", False),  # existing SPDX identifiers, but missing in fake_fedora_licenses
    )
)
def test_compliance_check_with_fedora(identifier, expected, fake_fedora_licenses):
    assert _is_compliant_with_fedora(identifier, fake_fedora_licenses) is expected


@pytest.mark.parametrize(
    ("identifiers", "expected"),
    (
        (["AAL", "MIT", "CECILL-B"], (True, [])),
        ([], (False, [])),
        (["LicenseRef-LPPL", "MIT"], (False, ["LicenseRef-LPPL"])),  # mixed not allowed/allowed
        (["LicenseRef-LPPL", "Aladdin", "LicenseRef-OpenFlow", "APL-1.0"], (False, ["LicenseRef-LPPL", "Aladdin", "LicenseRef-OpenFlow", "APL-1.0"])),
    ),
)
def test_is_allowed_in_fedora(identifiers, expected, fake_fedora_licenses):
    """Test the function logic, bear in mind that Fedora status may change in time"""

    assert good_for_fedora(identifiers, licenses_dict=fake_fedora_licenses) == expected



"""Test the functionality of pyp2conf part of the tool.
The data is downloaded from the PyPI and stored in betamax casettes
to prevent loading from the internet on each request.
"""
import pytest

try:
    import tomllib
except ImportError:
    import tomli as tomllib

from pyp2spec.pyp2conf import PypiPackage, create_config_contents
from pyp2spec.pyp2conf import convert_version_to_rpm_scheme, NoLicenseDetectedError, PackageNotFoundError


def test_non_existent_package(betamax_session):
    with pytest.raises(PackageNotFoundError):
        PypiPackage("definitely-nonexisting-package-name", session=betamax_session)


@pytest.mark.parametrize("package, version",
    [
        ("aionotion", "2.0.3"),
        ("click", "7.1.2"),
    ]
)
def test_automatically_generated_config_is_valid(betamax_parametrized_session, package, version):
    """Run the config rendering in fully automated mode and compare the results"""
    config = create_config_contents(
        package=package,
        version=version,
        session=betamax_parametrized_session,
    )

    with open(f"tests/test_configs/default_python-{package}.conf", "rb") as config_file:
        loaded_contents = tomllib.load(config_file)

    assert config == loaded_contents


@pytest.mark.parametrize("package, version, alt_python",
    [
        ("Pello", "1.0.4", "3.9"),
    ]
)
def test_automatically_generated_config_with_alt_python_is_valid(
        betamax_parametrized_session, package, version, alt_python
    ):
    """Run the config rendering in fully automated mode and compare the results"""
    config = create_config_contents(
        package=package,
        version=version,
        python_alt_version=alt_python,
        session=betamax_parametrized_session,
    )

    with open(f"tests/test_configs/default_python{alt_python}-{package.lower()}.conf", "rb") as config_file:
        loaded_contents = tomllib.load(config_file)

    assert config == loaded_contents


def test_config_with_customization_is_valid(betamax_session):
    """Get the upstream metadata and modify some fields to get the custom config file.
    This also tests the compliance with Fedora Legal data by making
    a request to the remote resource.
    """
    package = "aionotion"
    config = create_config_contents(
        package=package,
        description="A asyncio-friendly library for Notion Home Monitoring devices.\n",
        version="2.0.3",
        summary="Python library for Notion Home Monitoring",
        top_level=True,
        compliant=True,
        session=betamax_session,
    )

    with open(f"tests/test_configs/customized_{package}.conf", "rb") as config_file:
        loaded_contents = tomllib.load(config_file)

    assert config == loaded_contents


def test_archful_package(betamax_session):
    """Generate config for numpy which is archful"""
    package = "numpy"
    config = create_config_contents(
        package=package,
        license="fake-string",
        version="1.25.2",
        session=betamax_session,
    )

    with open(f"tests/test_configs/default_python-{package}.conf", "rb") as config_file:
        loaded_contents = tomllib.load(config_file)

    assert config["archful"] == loaded_contents["archful"]


def test_package_with_extras(betamax_session):
    package = "sphinx"
    config = create_config_contents(
        package=package,
        license="fake-license",
        session=betamax_session,
    )

    with open(f"tests/test_configs/customized_python-{package}.conf", "rb") as config_file:
        loaded_contents = tomllib.load(config_file)

    assert config["extras"] == loaded_contents["extras"]


def test_license_classifier_read_correctly():
    fake_pkg_data = {"info":{"classifiers": [
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Operating System :: POSIX :: Linux",
        "License :: OSI Approved :: MIT License",
        "License :: OSI Approved :: MIT No Attribution License (MIT-0)",
        "Development Status :: 3 - Alpha",
    ]}}

    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    assert pkg.filter_license_classifiers() == [
        "License :: OSI Approved :: MIT License",
        "License :: OSI Approved :: MIT No Attribution License (MIT-0)"
    ]


def test_no_license_classifiers_and_no_license_keyword():
    pkg = PypiPackage("_", version="1.2.3", package_metadata={"info":{"classifiers": [], "license": ""}})
    assert pkg.filter_license_classifiers() == []
    with pytest.raises(NoLicenseDetectedError):
        pkg.license()


def test_compliant_license_is_returned(fake_fedora_licenses):
    fake_pkg_data = {"info": {"classifiers" : [
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Operating System :: POSIX :: Linux",
        "License :: OSI Approved :: MIT License",
        "License :: OSI Approved :: MIT No Attribution License (MIT-0)",
        "Development Status :: 3 - Alpha",
    ]}}

    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    assert pkg.license(check_compliance=True, licenses_dict=fake_fedora_licenses) == "MIT AND MIT-0"


def test_bad_license_fails_compliance_check(fake_fedora_licenses):
    fake_pkg_data = {"info": {"classifiers" : [
        "License :: OSI Approved :: European Union Public Licence 1.0 (EUPL 1.0)"
    ]}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)

    with pytest.raises(NoLicenseDetectedError):
        pkg.license(check_compliance=True, licenses_dict=fake_fedora_licenses)


def test_bad_license_without_compliance_check_is_returned():
    fake_pkg_data = {"info": {"classifiers" : [
        "License :: OSI Approved :: European Union Public Licence 1.0 (EUPL 1.0)"
    ]}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)

    assert pkg.license(check_compliance=False) == "EUPL-1.0"


def test_mix_good_bad_licenses_fail_compliance_check(fake_fedora_licenses):
    fake_pkg_data = {"info": {"classifiers" : [
        "License :: OSI Approved :: European Union Public Licence 1.0 (EUPL 1.0)",
        "License :: OSI Approved :: MIT License",
    ]}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    with pytest.raises(NoLicenseDetectedError):
        pkg.license(check_compliance=True, licenses_dict=fake_fedora_licenses)


def test_mix_good_bad_licenses_without_compliance_check_are_returned():
    fake_pkg_data = {"info": {"classifiers" : [
        "License :: OSI Approved :: European Union Public Licence 1.0 (EUPL 1.0)",
        "License :: OSI Approved :: MIT License",
    ]}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    assert pkg.license(check_compliance=False) == "EUPL-1.0 AND MIT"


def test_license_keyword_without_compliance_check():
    fake_pkg_data = {"info": {"license": "BSD-2-Clause", "classifiers": []}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    assert pkg.license(check_compliance=False) == "BSD-2-Clause"


def test_license_keyword_with_compliance_check(fake_fedora_licenses):
    fake_pkg_data = {"info": {"license": "RSCPL", "classifiers": []}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    with pytest.raises(NoLicenseDetectedError):
        pkg.license(check_compliance=True, licenses_dict=fake_fedora_licenses)


@pytest.mark.parametrize("compliant", (True, False))
def test_empty_license_keyword_fails(compliant, fake_fedora_licenses):
    fake_pkg_data = {"info": {"license": "", "classifiers": []}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    with pytest.raises(NoLicenseDetectedError):
        pkg.license(check_compliance=compliant, licenses_dict=fake_fedora_licenses)


def test_zip_sdist_is_added_to_source_macro():

    fake_pkg_data = {
        "urls": [{
            "filename": "Awesome_TestPkg-1.2.3.zip",
            "packagetype": "sdist",
        }],
    }
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    assert pkg.source() == "%{pypi_source Awesome_TestPkg %{version} zip}"


def test_no_homepage_in_metadata():
    fake_pkg_data = {"info": {"package_url": "https://foo"}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    assert pkg.project_url() == "https://foo"


def test_summary_is_generated_if_not_in_upstream():
    fake_pkg_data = {"info": {"summary": ""}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    assert pkg.summary() == "..."


def test_summary_is_generated_if_upstream_data_is_multiline():
    fake_pkg_data = {"info": {"summary": "I\nforgot\nthat summary\nmust\nbe short"}}
    pkg = PypiPackage("_", version="1.2.3", package_metadata=fake_pkg_data)
    assert pkg.summary() == "..."


def test_capitalized_underscored_pypi_name_is_normalized():
    fake_pkg_data = {"info": {"name": "Awesome_TestPkg"}}
    pkg = PypiPackage("Awesome_TestPkg", version="1.2.3",package_metadata=fake_pkg_data)
    assert pkg.pypi_name == "awesome-testpkg"
    assert pkg.python_name() == "python-awesome-testpkg"


@pytest.mark.parametrize(
    ("pypi_name", "alt_version", "expected"), [
        ("foo", "3.10", "python3.10-foo"),
        ("python-foo", "3.9", "python3.9-foo"),
        ("python_foo", "3.12", "python3.12-foo"),
        ("foo", None, "python-foo"),
        ("python-foo", None, "python-foo"),
        ("python_foo", None, "python-foo"),
    ]
)
def test_python_name(pypi_name, alt_version, expected):
    fake_pkg_data = {"info": {"name": pypi_name}}
    pkg = PypiPackage(pypi_name, version="1.2.3", package_metadata=fake_pkg_data)
    assert pkg.python_name(python_alt_version=alt_version) == expected


@pytest.mark.parametrize(
    ("pypi_version", "rpm_version"), [
        ("1.1.0", "1.1.0"),
        ("1!0.2.13", "1:0.2.13"),
        ("0.0.2-beta1", "0.0.2~b1"),
        ("0.5.40-0", "0.5.40^post0"),
    ]
)
def test_pypi_version_is_converted_to_rpm(pypi_version, rpm_version):
    assert convert_version_to_rpm_scheme(pypi_version) == rpm_version


@pytest.mark.parametrize(
    ("pypi_version", "pypi_version_macro"), [
        ("1.1.0", "%{version}"),  # conversion to RPM doesn't change the string
        ("1!0.2.13", "1!0.2.13"),  # Version string is normalized, see previous test
        ("0.0.2-beta1", "0.0.2-beta1"),
        ("0.5.40-0", "0.5.40-0"),
    ]
)
def test_pypi_version_or_macro(pypi_version, pypi_version_macro):
    # to prevent tests sending requests to PyPI
    fake_pkg_data = {
        "info": "intentionally not empty package metadata"
    }
    pkg = PypiPackage("_", version=pypi_version, package_metadata=fake_pkg_data)
    assert pkg.pypi_version_or_macro() == pypi_version_macro


def test_extras_detected_correctly():
    fake_pkg_data = {
        "info": {
            "requires_dist": [
                "foo ; platform_system == 'Windows'",
                "bar ; python_version < '3.8'",
                "baz",
                "foobar ; extra == 'docs'",
                "foobaz>=5.3.1",
                "spam>=3.5.0 ; extra == 'lint'",
                "ham[eggs]>=0.10; extra == 'test'",
                "eggs>=2.12; implementation_name != 'pypy' and extra == 'dev'",
            ]
        }
    }
    pkg = PypiPackage("_", version="0", package_metadata=fake_pkg_data)
    assert pkg.extras() == ["dev", "docs", "lint", "test"]


@pytest.mark.parametrize(
    ("wheel_name", "archful"), [
        ("sampleproject-3.0.0-py3-none-any.whl", False),
        ("numpy-1.26.0-cp39-cp39-win_amd64.whl", True),
        ("numpy-1.26.0-cp312-cp312-manylinux_2_17_x86_64.manylinux2014_x86_64.whl", True),
        ("numpy-1.26.0-cp39-cp39-musllinux_1_1_x86_64.whl", True),
    ]
)
def test_archfulness_is_detected(wheel_name, archful):
    fake_pkg_data = {
        "urls": [
            {
                "packagetype": "bdist_wheel",
                "filename": wheel_name,
            }
        ]
    }
    pkg = PypiPackage("_", version="0", package_metadata=fake_pkg_data)
    assert pkg.is_archful() is archful


def test_archfulness_is_detected_from_multiple_urls_1():
    fake_pkg_data = {
        "urls": [
            {
                "packagetype": "sdist",
                "filename": "sampleproject-3.0.0.tar.gz",
            },
            {
                "packagetype": "bdist_wheel",
                "filename": "sampleproject-3.0.0-py3-none-any.whl",
            },
        ]
    }
    pkg = PypiPackage("_", version="0", package_metadata=fake_pkg_data)
    assert not pkg.is_archful()


def test_archfulness_is_detected_from_multiple_urls_2():
    fake_pkg_data = {
        "urls": [
            {
                "packagetype": "bdist_wheel",
                "filename": "sampleproject-3.0.0-py3-none-any.whl",
            },
            {
                "packagetype": "bdist_wheel",
                "filename": "sampleproject-3.0.0-cp312-abi3-manylinux1_x86_64.whl",
            },

        ]
    }
    pkg = PypiPackage("_", version="0", package_metadata=fake_pkg_data)
    assert pkg.is_archful()


def test_archfulness_is_detected_from_multiple_urls_3():
    fake_pkg_data = {
        "urls": [
            {
                "packagetype": "bdist_wheel",
                "filename": "sampleproject-3.0.0-cp312-abi3-manylinux1_x86_64.whl",
            },
            {
                "packagetype": "bdist_wheel",
                "filename": "sampleproject-3.0.0-cp39-cp39-win_amd64.whl",
            },
            {
                "packagetype": "bdist_wheel",
                "filename": "sampleproject-3.0.0-py3-none-any.whl",
            },
        ]
    }
    pkg = PypiPackage("_", version="0", package_metadata=fake_pkg_data)
    assert pkg.is_archful()
